@@ -2,12 +2,12 @@
 //! Crates APIs, essential structs, functions, methods are all here!
 //!
 //! To quickly understand how to use this crate, have a look at the
-//! documentation for `bitcoin_explorer::api::BitcoinDB`!!.
+//! documentation for `bitcoin_explorer::BitcoinDB`!!.
 //!
 //! # Example
 //!
 //! ```rust
-//! use bitcoin_explorer::api::BitcoinDB;
+//! use bitcoin_explorer::BitcoinDB;
 //! use std::path::Path;
 //!
 //! let path = Path::new("/Users/me/bitcoin").unwrap();
@@ -28,11 +28,11 @@ use crate::parser::script::{evaluate_script, ScriptInfo};
 use crate::parser::tx_index::TxDB;
 use std::path::Path;
 // re-exports
-pub use crate::api::connected::{ConnectedBlockIteratorFull, ConnectedBlockIteratorSimple};
-pub use crate::iter::{BlockIterator, ConnectedBlockIterator};
+pub use crate::iter::{BlockIter, ConnectedBlockIter};
 pub use crate::parser::block_index::{BlockIndex, BlockIndexRecord};
 pub use crate::parser::proto::connected_proto::{
-    TxConnectable, FConnectedBlock, FConnectedTransaction, SConnectedBlock, SConnectedTransaction,
+    BlockConnectable, FConnectedBlock, FConnectedTransaction, SConnectedBlock,
+    SConnectedTransaction, TxConnectable,
 };
 pub use crate::parser::proto::full_proto::{FBlock, FBlockHeader, FTransaction, FTxOut};
 pub use crate::parser::proto::simple_proto::{SBlock, SBlockHeader, STransaction, STxOut};
@@ -70,7 +70,7 @@ impl BitcoinDB {
     /// # Example
     ///
     /// ```rust
-    /// use bitcoin_explorer::api::BitcoinDB;
+    /// use bitcoin_explorer::BitcoinDB;
     /// use std::path::Path;
     ///
     /// let path = Path::new("/Users/me/bitcoin").unwrap();
@@ -105,8 +105,7 @@ impl BitcoinDB {
     /// Get the maximum height found in block index.
     ///
     /// It is not necessarily true that any height smaller than
-    /// `get_max_height()` can be used to query `get_block()`,
-    /// `get_block_full()` or `get_block_simple()`.
+    /// `get_max_height()` can be used to query `get_block()`.
     ///
     /// This max_height is only the max height of block header
     /// information. The actual block data might not have been
@@ -128,7 +127,7 @@ impl BitcoinDB {
     ///
     /// ## Compute total number of transactions
     /// ```rust
-    /// use bitcoin_explorer::api::BitcoinDB;
+    /// use bitcoin_explorer::BitcoinDB;
     /// use std::path::Path;
     ///
     /// let path = Path::new("/Users/me/bitcoin").unwrap();
@@ -181,7 +180,7 @@ impl BitcoinDB {
     ///
     /// # Example
     /// ```rust
-    /// use bitcoin_explorer::api::{BitcoinDB, FBlock, SBlock, Block};
+    /// use bitcoin_explorer::{BitcoinDB, FBlock, SBlock, Block};
     /// use std::path::Path;
     ///
     /// let path = Path::new("/Users/me/bitcoin").unwrap();
@@ -211,7 +210,7 @@ impl BitcoinDB {
     ///
     /// # Example
     /// ```rust
-    /// use bitcoin_explorer::api::{BitcoinDB, Transaction, FTransaction, STransaction, Txid, FromHex};
+    /// use bitcoin_explorer::{BitcoinDB, Transaction, FTransaction, STransaction, Txid, FromHex};
     /// use std::path::Path;
     ///
     /// let path = Path::new("/Users/me/bitcoin").unwrap();
@@ -277,7 +276,7 @@ impl BitcoinDB {
     /// # Example
     ///
     /// ```rust
-    /// use bitcoin_explorer::api::{BitcoinDB, Block, SBlock, FBlock};
+    /// use bitcoin_explorer::{BitcoinDB, Block, SBlock, FBlock};
     /// use std::path::Path;
     ///
     /// let path = Path::new("/Users/me/bitcoin").unwrap();
@@ -307,11 +306,11 @@ impl BitcoinDB {
     /// }
     /// ```
     ///
-    pub fn iter_block<T>(&self, start: u32, end: u32) -> BlockIterator<T>
+    pub fn iter_block<T>(&self, start: u32, end: u32) -> BlockIter<T>
     where
         T: From<Block> + Send + 'static,
     {
-        BlockIterator::from_range(self, start, end)
+        BlockIter::from_range(self, start, end)
     }
 
     ///
@@ -337,7 +336,7 @@ impl BitcoinDB {
     /// # Example
     ///
     /// ```rust
-    /// use bitcoin_explorer::api::{BitcoinDB, Block, FBlock, SBlock};
+    /// use bitcoin_explorer::{BitcoinDB, Block, FBlock, SBlock};
     /// use std::path::Path;
     ///
     /// let path = Path::new("/Users/me/bitcoin").unwrap();
@@ -371,10 +370,7 @@ impl BitcoinDB {
     ///
     /// ```
     ///
-    pub fn iter_heights<T: 'static + From<Block> + Send>(
-        &self,
-        heights: Vec<u32>,
-    ) -> BlockIterator<T> {
-        BlockIterator::new(self, heights)
+    pub fn iter_heights<T: 'static + From<Block> + Send>(&self, heights: Vec<u32>) -> BlockIter<T> {
+        BlockIter::new(self, heights)
     }
 }
