@@ -3,7 +3,7 @@ use crate::iter::fetch_connected_async::{fetch_block_connected, TaskConnected};
 use crate::iter::util::{DBCopy, VecMap};
 use crate::parser::proto::connected_proto::{BlockConnectable, TxConnectable};
 use std::borrow::BorrowMut;
-use std::collections::{BTreeMap, VecDeque};
+use std::collections::{HashMap, VecDeque};
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::mpsc::{sync_channel, Receiver};
 use std::sync::{Arc, Condvar, Mutex};
@@ -47,8 +47,8 @@ where
         let error_state_copy = error_state.clone();
         let (sender, receiver) = sync_channel(cpus * 10);
         let unspent: Arc<
-            Mutex<BTreeMap<Txid, Arc<Mutex<VecMap<<TBlock::Tx as TxConnectable>::TOut>>>>>,
-        > = Arc::new(Mutex::new(BTreeMap::new()));
+            Mutex<HashMap<Txid, Arc<Mutex<VecMap<<TBlock::Tx as TxConnectable>::TOut>>>>>,
+        > = Arc::new(Mutex::new(HashMap::new()));
         let db = DBCopy::from_bitcoin_db(db);
         // worker master
         let worker_thread = thread::spawn(move || {
