@@ -16,16 +16,10 @@
 //!
 //! Currently it only supports all standard script types.
 //!
-//! ## Rust vs Python
-//!
-//! For `Rust` users. Please use `bitcoin_explorer::api`.
-//! The structures and functions defined in `lib` here
-//! are intended for `python` users.
-//!
 //! # Example
 //!
 //! ```rust
-//! use bitcoin_explorer::api::BitcoinDB;
+//! use bitcoin_explorer::BitcoinDB;
 //! use std::path::Path;
 //!
 //! let path = Path::new("/Users/me/bitcoin").unwrap();
@@ -42,6 +36,7 @@ pub mod api;
 pub mod iter;
 pub mod parser;
 
+#[doc(inline)]
 pub use crate::api::*;
 use pyo3::prelude::*;
 use pyo3::PyIterProtocol;
@@ -83,7 +78,7 @@ impl BitcoinDBPy {
 
     #[pyo3(text_signature = "($self, height, /)")]
     fn get_block_full_connected(&self, height: i32, py: Python) -> PyResult<PyObject> {
-        match self.db.get_block_connected::<FConnectedBlock>(height) {
+        match self.db.get_connected_block::<FConnectedBlock>(height) {
             Ok(block) => Ok(pythonize(py, &block)?),
             Err(e) => Err(pyo3::exceptions::PyException::new_err(e.to_string())),
         }
@@ -91,7 +86,7 @@ impl BitcoinDBPy {
 
     #[pyo3(text_signature = "($self, height, /)")]
     fn get_block_simple_connected(&self, height: i32, py: Python) -> PyResult<PyObject> {
-        match self.db.get_block_connected::<SConnectedBlock>(height) {
+        match self.db.get_connected_block::<SConnectedBlock>(height) {
             Ok(block) => Ok(pythonize(py, &block)?),
             Err(e) => Err(pyo3::exceptions::PyException::new_err(e.to_string())),
         }
@@ -166,7 +161,10 @@ impl BitcoinDBPy {
     #[pyo3(text_signature = "($self, txid, /)")]
     fn get_transaction_full_connected(&self, txid: String, py: Python) -> PyResult<PyObject> {
         if let Ok(txid) = Txid::from_hex(&txid) {
-            match self.db.get_transaction_connected::<FConnectedTransaction>(&txid) {
+            match self
+                .db
+                .get_connected_transaction::<FConnectedTransaction>(&txid)
+            {
                 Ok(t) => Ok(pythonize(py, &t)?),
                 Err(e) => Err(pyo3::exceptions::PyException::new_err(e.to_string())),
             }
@@ -180,7 +178,10 @@ impl BitcoinDBPy {
     #[pyo3(text_signature = "($self, txid, /)")]
     fn get_transaction_simple_connected(&self, txid: String, py: Python) -> PyResult<PyObject> {
         if let Ok(txid) = Txid::from_hex(&txid) {
-            match self.db.get_transaction_connected::<SConnectedTransaction>(&txid) {
+            match self
+                .db
+                .get_connected_transaction::<SConnectedTransaction>(&txid)
+            {
                 Ok(t) => Ok(pythonize(py, &t)?),
                 Err(e) => Err(pyo3::exceptions::PyException::new_err(e.to_string())),
             }
@@ -202,13 +203,13 @@ impl BitcoinDBPy {
     }
 
     #[pyo3(text_signature = "($self, start, stop, /)")]
-    fn iter_block_full_seq(&self, start: u32, stop: u32) -> PyResult<FBlockIteratorSequential> {
-        Ok(FBlockIteratorSequential::new(&self.db, start, stop))
+    fn iter_block_full_seq(&self, start: u32, stop: u32) -> PyResult<FBlockIterator> {
+        Ok(FBlockIterator::new(&self.db, start, stop))
     }
 
     #[pyo3(text_signature = "($self, start, stop, /)")]
-    fn iter_block_simple_seq(&self, start: u32, stop: u32) -> PyResult<SBlockIteratorSequential> {
-        Ok(SBlockIteratorSequential::new(&self.db, start, stop))
+    fn iter_block_simple_seq(&self, start: u32, stop: u32) -> PyResult<SBlockIterator> {
+        Ok(SBlockIterator::new(&self.db, start, stop))
     }
 
     #[pyo3(text_signature = "($self, stop, /)")]
@@ -239,214 +240,91 @@ impl BitcoinDBPy {
     }
 }
 
-#[pyclass]
-struct SBlockIteratorArray {
-    iter: BlockIterator<SBlock>,
-}
+// construct python iterators
+derive_py_iter!(
+    FBlockIteratorArray,
+    BlockIter,
+    FBlock,
+    iter_heights,
+    heights: Vec<u32>
+);
+derive_py_iter!(
+    SBlockIteratorArray,
+    BlockIter,
+    SBlock,
+    iter_heights,
+    heights: Vec<u32>
+);
+derive_py_iter!(
+    FBlockIterator,
+    BlockIter,
+    FBlock,
+    iter_block,
+    start: u32,
+    end: u32
+);
+derive_py_iter!(
+    SBlockIterator,
+    BlockIter,
+    SBlock,
+    iter_block,
+    start: u32,
+    end: u32
+);
+derive_py_iter!(
+    FConnectedBlockIterator,
+    ConnectedBlockIter,
+    FConnectedBlock,
+    iter_connected_block,
+    end: u32
+);
+derive_py_iter!(
+    SConnectedBlockIterator,
+    ConnectedBlockIter,
+    SConnectedBlock,
+    iter_connected_block,
+    end: u32
+);
 
-impl SBlockIteratorArray {
-    fn new(db: &api::BitcoinDB, heights: Vec<u32>) -> Self {
-        SBlockIteratorArray {
-            iter: db.iter_heights::<SBlock>(heights),
+#[macro_export]
+macro_rules! derive_py_iter {
+    ($new_iter_type:ident, $inner_iter_type:ident, $iter_type:ty,
+     $new:ident, $( $p:ident: $type:ty ),*) => {
+        #[pyclass]
+        struct $new_iter_type {
+            iter: $inner_iter_type<$iter_type>,
         }
-    }
-}
 
-#[pyproto]
-impl PyIterProtocol for SBlockIteratorArray {
-    fn __iter__(slf: PyRef<Self>) -> PyRef<Self> {
-        slf
-    }
-
-    fn __next__(mut slf: PyRefMut<Self>) -> Option<PyObject> {
-        let option_block: Option<SBlock> = slf.iter.next();
-        if let Some(output) = option_block {
-            let gil_guard = Python::acquire_gil();
-            let py = gil_guard.python();
-            if let Ok(py_obj) = pythonize(py, &output) {
-                Some(py_obj)
-            } else {
-                None
+        impl $new_iter_type {
+            fn new(db: &api::BitcoinDB, $($p: $type),*) -> $new_iter_type {
+                $new_iter_type {
+                    iter: db.$new::<$iter_type>($($p),*),
+                }
             }
-        } else {
-            None
         }
-    }
-}
 
-#[pyclass]
-struct FBlockIteratorArray {
-    iter: BlockIterator<FBlock>,
-}
-
-impl FBlockIteratorArray {
-    fn new(db: &api::BitcoinDB, heights: Vec<u32>) -> Self {
-        FBlockIteratorArray {
-            iter: db.iter_heights::<FBlock>(heights),
-        }
-    }
-}
-
-#[pyproto]
-impl PyIterProtocol for FBlockIteratorArray {
-    fn __iter__(slf: PyRef<Self>) -> PyRef<Self> {
-        slf
-    }
-
-    fn __next__(mut slf: PyRefMut<Self>) -> Option<PyObject> {
-        let option_block: Option<FBlock> = slf.iter.next();
-        if let Some(output) = option_block {
-            let gil_guard = Python::acquire_gil();
-            let py = gil_guard.python();
-            if let Ok(py_obj) = pythonize(py, &output) {
-                Some(py_obj)
-            } else {
-                None
+        #[pyproto]
+        impl PyIterProtocol for $new_iter_type {
+            fn __iter__(slf: PyRef<Self>) -> PyRef<Self> {
+                slf
             }
-        } else {
-            None
-        }
-    }
-}
 
-#[pyclass]
-struct FBlockIteratorSequential {
-    iter: BlockIterator<FBlock>,
-}
-
-impl FBlockIteratorSequential {
-    fn new(db: &api::BitcoinDB, start: u32, end: u32) -> Self {
-        FBlockIteratorSequential {
-            iter: db.iter_block::<FBlock>(start, end),
-        }
-    }
-}
-
-#[pyproto]
-impl PyIterProtocol for FBlockIteratorSequential {
-    fn __iter__(slf: PyRef<Self>) -> PyRef<Self> {
-        slf
-    }
-
-    fn __next__(mut slf: PyRefMut<Self>) -> Option<PyObject> {
-        let option_block: Option<FBlock> = slf.iter.next();
-        if let Some(output) = option_block {
-            let gil_guard = Python::acquire_gil();
-            let py = gil_guard.python();
-            if let Ok(py_obj) = pythonize(py, &output) {
-                Some(py_obj)
-            } else {
-                None
+            fn __next__(mut slf: PyRefMut<Self>) -> Option<PyObject> {
+                let option_block: Option<$iter_type> = slf.iter.next();
+                if let Some(output) = option_block {
+                    let gil_guard = Python::acquire_gil();
+                    let py = gil_guard.python();
+                    if let Ok(py_obj) = pythonize(py, &output) {
+                        Some(py_obj)
+                    } else {
+                        None
+                    }
+                } else {
+                    None
+                }
             }
-        } else {
-            None
         }
-    }
-}
-
-#[pyclass]
-struct SBlockIteratorSequential {
-    iter: BlockIterator<SBlock>,
-}
-
-impl SBlockIteratorSequential {
-    fn new(db: &api::BitcoinDB, start: u32, end: u32) -> SBlockIteratorSequential {
-        SBlockIteratorSequential {
-            iter: db.iter_block::<SBlock>(start, end),
-        }
-    }
-}
-
-#[pyproto]
-impl PyIterProtocol for SBlockIteratorSequential {
-    fn __iter__(slf: PyRef<Self>) -> PyRef<Self> {
-        slf
-    }
-
-    fn __next__(mut slf: PyRefMut<Self>) -> Option<PyObject> {
-        let option_block: Option<SBlock> = slf.iter.next();
-        if let Some(output) = option_block {
-            let gil_guard = Python::acquire_gil();
-            let py = gil_guard.python();
-            if let Ok(py_obj) = pythonize(py, &output) {
-                Some(py_obj)
-            } else {
-                None
-            }
-        } else {
-            None
-        }
-    }
-}
-
-#[pyclass]
-struct FConnectedBlockIterator {
-    iter: api::ConnectedBlockIteratorFull,
-}
-
-impl FConnectedBlockIterator {
-    fn new(db: &api::BitcoinDB, end: u32) -> FConnectedBlockIterator {
-        FConnectedBlockIterator {
-            iter: db.iter_block_full_connected(end),
-        }
-    }
-}
-
-#[pyproto]
-impl PyIterProtocol for FConnectedBlockIterator {
-    fn __iter__(slf: PyRef<Self>) -> PyRef<Self> {
-        slf
-    }
-
-    fn __next__(mut slf: PyRefMut<Self>) -> Option<PyObject> {
-        let option_block: Option<FConnectedBlock> = slf.iter.next();
-        if let Some(output) = option_block {
-            let gil_guard = Python::acquire_gil();
-            let py = gil_guard.python();
-            if let Ok(py_obj) = pythonize(py, &output) {
-                Some(py_obj)
-            } else {
-                None
-            }
-        } else {
-            None
-        }
-    }
-}
-
-#[pyclass]
-struct SConnectedBlockIterator {
-    iter: api::ConnectedBlockIteratorSimple,
-}
-
-impl SConnectedBlockIterator {
-    fn new(db: &api::BitcoinDB, end: u32) -> SConnectedBlockIterator {
-        SConnectedBlockIterator {
-            iter: db.iter_block_simple_connected(end),
-        }
-    }
-}
-
-#[pyproto]
-impl PyIterProtocol for SConnectedBlockIterator {
-    fn __iter__(slf: PyRef<Self>) -> PyRef<Self> {
-        slf
-    }
-
-    fn __next__(mut slf: PyRefMut<Self>) -> Option<PyObject> {
-        let option_block: Option<SConnectedBlock> = slf.iter.next();
-        if let Some(output) = option_block {
-            let gil_guard = Python::acquire_gil();
-            let py = gil_guard.python();
-            if let Ok(py_obj) = pythonize(py, &output) {
-                Some(py_obj)
-            } else {
-                None
-            }
-        } else {
-            None
-        }
-    }
+    };
 }
 
 #[pymodule]
